/****************************************************************************
*  This file is part of the blinky program for the TNKernel RTOS
*
*  Copyright (c) 2009 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
*****************************************************************************
*  History:
*
*  20.09.2009  mifi   First Version, tested with a HITEX STR7 board
****************************************************************************/
#define __HARDWARE_C__

/*=========================================================================*/
/*  Includes                                                               */
/*=========================================================================*/
#include "arch.h"
#include "hardware.h"
#include "tn.h"

/*=========================================================================*/
/*  DEFINE: All Structures and Common Constants                            */
/*=========================================================================*/
#define PERIPHERAL_CLK     24000000
#define TIMER3_ID          21
#define TIMER3_PRESCALER   23
#define TIMER3_LOAD_VAL    ((PERIPHERAL_CLK / (TIMER3_PRESCALER + 1)) / 1000)

/*
 * On the MCBSTR7 we have 8 LEDs, connected to P2.8 ... P2.15
 */
#define LED_MASK  0xFF00
#define LED1      0x0100

#define LED_ON    IO0CLR
#define LED_OFF   IO0SET

/*=========================================================================*/
/*  DEFINE: Prototypes                                                     */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Data                                   */
/*=========================================================================*/

/*=========================================================================*/
/*  DEFINE: Definition of all local Procedures                             */
/*=========================================================================*/

/***************************************************************************/
/*  Timer3IRQHandler                                                       */
/*                                                                         */
/*  For the purpose of calculating timeouts and delays, TNKernel uses a    */
/*  time tick timer. In TNKernel, this time tick timer must to be a some   */
/*  kind of hardware timer that produces interrupt for time ticks          */
/*  processing. The period of this timer is user determined (usually in    */
/*  the range 0.5...20 ms). Within the time ticks interrupt processing,    */
/*  it is only necessary to call the tn_tick_int_processing() functions    */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void Timer3IRQHandler (void)
{
   TIM3_SR  &= ~0x4000;	         /* Clear the OCFA bit of Timer3 */
   TIM3_OCAR = TIMER3_LOAD_VAL;  /* Reload the Timer3 period register */
   TIM3_CNTR = 0xFFFC;           /* Reset the counter */
   
   /* Call the TNKernel time tick function */
   tn_tick_int_processing();
   
} /* Timer3IRQHandler */

/***************************************************************************/
/*  LEDInit                                                                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDInit (void)
{
   /*
    * Set the LED output pins to Push-Pull
    */
   IOPORT2_PC0 |=  LED_MASK;
   IOPORT2_PC1 &= ~LED_MASK;
   IOPORT2_PC2 |=  LED_MASK;
   IOPORT2_PD  &= ~LED_MASK;

   LedOff();
} /* LEDInit */

/*=========================================================================*/
/*  DEFINE: All code exported                                              */
/*=========================================================================*/
/***************************************************************************/
/*  HardwareInit                                                           */
/*                                                                         */
/*  Init the hardware like the timer and the GPIO port for the LED.        */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void HardwareInit (void)
{
   /*
    * Init the interrupt system first
    */            
   tn_irq_init();
   
   /*
    * The CPU clock is set to 48MHz and the peripherals bus to 24MHz
    * by the startup code from CrossWorks for ARM. Here we must 
    * only init the timer. Setup the Timer3 with a period of 1ms.
    */

   TIM3_CR2  = 0x4000 | TIMER3_PRESCALER;    //  Set the Timer1 prescaler
   TIM3_CR1  = 0x8040;              // Timer1 - in Output Compare mode
   TIM3_OCAR = TIMER3_LOAD_VAL;     // Timer1 period
   TIM3_CNTR = 0xFFF0;              // Reset the counter & run it

   /* Install timer3 with prio 1 */
   tn_irq_install(TIMER3_ID, 1, Timer3IRQHandler);
   
   /*
    * Init the GPIO for the LED
    */
   LEDInit();

} /* HardwareInit */

/***************************************************************************/
/*  tn_cpu_int_enable                                                      */
/*                                                                         */
/*  The tn_cpu_int_enable() function enables all interrupts for vectors    */
/*  utilized by the user project and than enables global interrupts.       */
/*  The user must enable the system time ticks interrupt in this function. */
/*  TNKernel calls this function without user intervention.                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void tn_cpu_int_enable (void)
{
   /* Enable Timer3 interrupt */
   EIC_IER0 |= (1L << TIMER3_ID);
   
   /* 
    * Enable the Enhanced Interrupt Controller IRQ 
    * output to the CPU.   
    */
   EIC_ICR = EIC_ICR_IRQ_EN_MASK;   
   
   tn_arm_enable_interrupts();
} /* tn_cpu_int_enable */

/***************************************************************************/
/*  LedOn                                                                  */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void LedOn (void)
{
   IOPORT2_PD |= LED1;
} /* LedOn */

/***************************************************************************/
/*  LedOff                                                                 */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
void LedOff (void)
{
   IOPORT2_PD &= ~LED1;
} /* LedOff */

/*** EOF ***/
